#include <stdio.h>
#include "config.h"
#include "area.h"
#include "status.h"
#include "message.h"
#include "command.h"

#define SYMBOL_FLAG_NUM 2
typedef struct {
    noun_id_t nid;
    flag_id_t fids[SYMBOL_FLAG_NUM];
    char *description;
    char *message;
    char *hint;
} symbol_t;


typedef struct {
    int num;
    symbol_t *symbols;
} symbol_entry_t;
    
symbol_t death_symbols[] = {
    {ni_glasses,   
     {fi_death_by_touching_water, fi_invalid},
     "ヨシノ教授の眼鏡だ。浴槽の中身についてはあまり考えたくない。",
     "浴槽の水に触れて死す",
     "死を招くアヒル"},
    {ni_necklace,  
     {fi_death_by_touching_shadow, fi_invalid},
     "ヨシノ夫人の首飾りだ。",
     "2階の影に触れて死す",
     "準備無しに触れると死を招く"},
    {ni_armor,     
     {fi_death_by_armor, fi_invalid},
     "鎧のミニチュアだ。",
     "鎧に切られて死す",
     "真剣白刃取りは実装されていない"},
    {ni_slingshot,
     {fi_death_by_exiting_window, fi_invalid},
     "嫌な感じでぽっきり折れたぱちんこだ。",
     "窓に挟まれて死す",
     "いいか出るなよ、絶対出るなよ"},
    {ni_skull,
     {fi_death_by_starvation, fi_invalid},
     "誰かの髑髏だ。",
     "マクドナルドの轍を踏みて死す",
     "追われながら某所に入ると…"},
    {ni_tentacle,
     {fi_death_by_tentacle, fi_invalid},
     "触手だ。烏賊の下足ではない。",
     "支配者に捕らわれて死す",
     "戸口にあらわれたもの"},
};
#define DEATH_SYMBOL_NUM (sizeof(death_symbols)/sizeof(symbol_t))

symbol_t alternative_symbols[] = {
    {ni_fossil,
     {fi_clear_armor_alive, fi_clear_armor_dead},
     "ゴードリセラス科のアンモナイトの化石だ。"
     "おそらく日本の北海道産だろう。"
     "ノジュールで産出し保存状態が非常によいが、"
     "母岩がおそろしく硬くてクリーニングが大変である…"
     "今、なにかどうでもいい情報が頭に流れ込んできた気がする。",
     "鎧を倒して／倒さずにクリア",
     "鎧は倒さなくてもよい"},
    {ni_billiard_ball,
     {fi_clear_treasure_chest_unlocked, fi_clear_treasure_chest_locked},
     "ビリヤードボールだ。",
     "宝箱の錠を開けて／壊してクリア",
     "宝箱の鍵は取り忘れても問題ない"},
    {ni_portrait,
     {fi_clear_portrait_enabled, fi_clear_portrait_disabled},
     "目が動く肖像画だ。マクドナルド邸七不思議の一つらしい。",
     "肖像画を外して／外さずにクリア",
     "肖像画をよく観察すると…"},
    {ni_human_bone,
     {fi_clear_using_human_bone, fi_clear_using_chicken_bone},
     "誰かの大腿骨だ。",
     "人の骨／鶏の骨でサイクロトロン機関を起動してクリア",
     "カルシウムなら何でもいいんだよ"},
    {ni_telephone,
     {fi_clear_phone_call_answered, fi_clear_phone_call_ignored},
     "懐かしいダイアル式の電話だ。",
     "電話に出て／出ずにクリア",
     "電話をかけても誰も出んわ"},
};
#define ALTERNATIVE_SYMBOL_NUM (sizeof(alternative_symbols)/sizeof(symbol_t))

symbol_t complete_symbols[] = {
    {ni_lantern,   
     {fi_clear_map_completed, fi_invalid},
     "けっして油切れを起こさない不思議なランタンだ。",
     "マップ内をすべて回った",
     "くまなく歩け"},
    {ni_girl_doll,   
     {fi_clear_dummy_item_completed, fi_invalid},
     "意味ありげで意味のない人形だ。",
     "ダミーアイテムをすべて入手",
     "取れるものはなんでも取れ"},
    {ni_witch_doll,   
     {fi_clear_read_completed, fi_invalid},
     "いかにもダミーっぽい魔女の人形だ。",
     "文書をすべて読んだ",
     "読めるものはなんでも読め"},
    {ni_large_robot,   
     {fi_clear_character_completed, fi_invalid},
     "親の懐に厳しい、巨大な合体ロボットの玩具だ。",
     "関係者の姿をすべて見た。",
     "読めるものはなんでも読め"},
};
#define COMPLETE_SYMBOL_NUM (sizeof(complete_symbols)/sizeof(symbol_t))

symbol_t progress_symbols[] = {
    {ni_flowerpot,   
     {fi_death_entered_house, fi_invalid},
     "トリフィドの鉢植えだ。幸いにも枯れている。",
     "屋敷に入った",
     "なんとかして玄関の扉を開けよう"},
    {ni_fishing_rod,   
     {fi_death_entered_2f, fi_invalid},
     "教授の趣味の釣竿だ。",
     "2階に入った",
     "1階のどこかに鍵が…"},
    {ni_candle,   
     {fi_death_candle_lit, fi_invalid},
     "なかなか火がつかない蝋燭だ。",
     "交霊術に成功した",
     "食堂にて儀式を行うべし"},
    {ni_bottle,   
     {fi_death_magic_circle_invoked, fi_invalid},
     "謎めいた小瓶だ。中にはヤバそうな粉が入っている。",
     "魔法陣を起動した",
     "隠し部屋を見つけ儀式を行うべし"},
    {ni_amulet,   
     {fi_clear_game, fi_invalid},
     "旧神の印だ。",
     "エンディングを迎えた",
     "奴をなんとかして脱出しよう"},
};
#define PROGRESS_SYMBOL_NUM (sizeof(progress_symbols)/sizeof(symbol_t))

symbol_entry_t symbol_table[] = {
    {DEATH_SYMBOL_NUM, death_symbols},
    {ALTERNATIVE_SYMBOL_NUM, alternative_symbols},
    {COMPLETE_SYMBOL_NUM, complete_symbols},
    {PROGRESS_SYMBOL_NUM, progress_symbols},
};

#define SYMBOLS_NUM (sizeof(symbol_table)/sizeof(symbol_entry_t))

int is_achievements_on(flag_id_t fids[]);
int count_achievements(int num, symbol_t symbols[]);
void message_active_symbols(int num, symbol_t symbols[]);
void message_achievements_count(char *header, int num, symbol_t symbols[]);
char *achievements_hint(int num, symbol_t symbols[]);
flag_id_t *symbol_nid2fids(noun_id_t nid, int num, symbol_t symbols[]);
char *symbol_nid2message(noun_id_t nid, int num, symbol_t symbols[]);
void message_symbol(noun_id_t nid, int num, symbol_t symbols[]);
void tentacle_duck(void);
void tentacle_attack(void);
int command_symbols(verb_id_t vid, noun_id_t nid, int *exist);
    
int area_achievements(verb_id_t vid, noun_id_t nid, int *exist){
    assert(vid != vi_invalid);
	*exist = 1;
    switch(nid){
    case ni_invalid:
        switch(vid){
        case vi_look:
            DescribeArea;
			message("左右の壁際にはそれぞれ%nlと%nlがある。"
                    "正面には%nlがあり、その手前には%nlがある。",
					ni_cabinet, ni_table, ni_fireplace, ni_treasure_chest);
            if (Not(achievements_dog_sitting)){
                message("%nwの右側には%nlがあり、その前に%nlが立っている。",
                        ni_fireplace, ni_door, ni_dog);
            }
            else {
                message("%nnの右側には%nlがある。",
                        ni_fireplace, ni_door);
                message("床の上には%nlが座っている。",
                        ni_dog);
            }
            message("タイトル画面に戻るには、「%v」と入力してください。",
                    vi_backward);
            return 1;
        case vi_backward:
        case vi_down:
		case vi_exit:
            move_to_title();
            return 1;
        default:
            break;
        }
        break;
    case ni_table:
        switch (vid){
        case vi_look:
            message("これは死のテーブルだ。");
            message("テーブルの上や周りには\\");
            message_active_symbols(DEATH_SYMBOL_NUM, death_symbols);
            return 1;
        case vi_examine:
            message_achievements_count("死因", DEATH_SYMBOL_NUM, death_symbols);
            return 1;
        default:
            break;
        }
        break;
    case ni_fireplace:
        switch (vid){
        case vi_look:
            message("これは選択の暖炉だ。");
            message("周りには\\");
            message_active_symbols(ALTERNATIVE_SYMBOL_NUM, alternative_symbols);
            return 1;
        case vi_examine:
            message_achievements_count("選択肢", ALTERNATIVE_SYMBOL_NUM, 
                                       alternative_symbols);
            return 1;
        default:
            break;
        }
        break;
    case ni_treasure_chest:
        switch (vid){
        case vi_look:
            message("これはやりこみの宝箱だ。");
            if (Is(achievements_treasure_chest_opened) &&
                Not(achievements_duck_moved))
            {
                message("箱の中には%nlが入っている。", ni_rubber_duck);
            }
            message("辺りには\\");
            message_active_symbols(COMPLETE_SYMBOL_NUM, complete_symbols);
            return 1;
        case vi_examine:
            message_achievements_count("やりこみ", COMPLETE_SYMBOL_NUM,
                                       complete_symbols);
            return 1;
        case vi_open:
            if (Not(achievements_treasure_chest_opened)){
                if (Not(achievements_treasure_chest_unlocked)){
                    message("鍵が掛かっている。");
                    return 1;
                }
                message("%nnを開けた。", nid);
                Set(achievements_treasure_chest_opened);
                if (Not(achievements_chicken_dropped)){
                    message("箱の中から%nlが出てきた。", ni_chicken);
                    Set(achievements_chicken_dropped);
                    message("%nnがしっぽを振って%nnに飛びついた。",
                            ni_dog, ni_chicken);
                    Set(achievements_dog_sitting);
                }
            }
            else {
                message("%nwはもう開いている。", nid);
            }
            return 1;
        case vi_close:
            if (Is(achievements_treasure_chest_opened)){
                message("%nwを閉めた。", nid);
                Reset(achievements_treasure_chest_opened);
            }
            else {
                message("%nwは閉まっている。", nid);
            }
            return 1;
        case vi_hit:
        case vi_kick:
        case vi_destroy:
            message("今回は鍵がないと開かないようだ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_cabinet:
        switch (vid){
        case vi_look:
            message("これは進捗のキャビネットだ。");
            message("キャビネットの上や周りには\\");
            message_active_symbols(PROGRESS_SYMBOL_NUM, progress_symbols);
            if (Is(achievements_cabinet_opened)){
                if (Not(achievements_wooden_key_taken) &&
                    Not(achievements_sponge_taken)){
                    message("中には%nlと%nlが入っている。",
                            ni_sponge, ni_wooden_key);
                }
                else if (Not(achievements_sponge_taken)){
                    message("中には%nlが入っている。", ni_sponge);
                }
                else if (Not(achievements_wooden_key_taken)){
                    message("中には%nlが入っている。", ni_wooden_key);
                }
                else {
                    message("中は空だ。");
                }
            }
            return 1;
        case vi_examine:
            message_achievements_count("進捗", PROGRESS_SYMBOL_NUM,
                                       progress_symbols);
            return 1;
		case vi_open:
            if (Not(achievements_cabinet_opened)){
                message("%nnを開けた。", ni_cabinet);
                Set(achievements_cabinet_opened);
            }
            else {
                message("%nnはもう開いている。", ni_cabinet);
            }
            return 1;
		case vi_close:
            if (Is(achievements_cabinet_opened)){
                message("%nnを閉めた。", ni_cabinet);
                Reset(achievements_cabinet_opened);
            }
            else {
                message("%nnは閉じている。", ni_cabinet);
            }
            return 1;
        default:
            break;
        }
        break;
        /*
    case ni_glasses:
    case ni_necklace:
    case ni_armor:
    case ni_slingshot:
    case ni_skull:
    case ni_tentacle:
        if (!is_achievements_on(symbol_nid2fids(nid, DEATH_SYMBOL_NUM,
                                                death_symbols)))
        {
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
            message_symbol(nid, DEATH_SYMBOL_NUM, death_symbols);
            return 1;
        default:
            break;
        }
        break;
    case ni_fossil:
    case ni_billiard_ball:
    case ni_human_bone:
    case ni_telephone:
    case ni_portrait:
        if (!is_achievements_on(symbol_nid2fids(nid, ALTERNATIVE_SYMBOL_NUM,
                                                alternative_symbols)))
        {
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
            message_symbol(nid, ALTERNATIVE_SYMBOL_NUM, alternative_symbols);
            return 1;
        default:
            break;
        }
        break;
    case ni_lantern:
    case ni_girl_doll:
    case ni_witch_doll:
        if (!is_achievements_on(symbol_nid2fids(nid, COMPLETE_SYMBOL_NUM,
                                                complete_symbols)))
        {
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
            message_symbol(nid, COMPLETE_SYMBOL_NUM, complete_symbols);
            return 1;
        default:
            break;
        }
        break;
    case ni_flowerpot:
    case ni_fishing_rod:
    case ni_candle:
    case ni_bottle:
    case ni_amulet:
        if (!is_achievements_on(symbol_nid2fids(nid, PROGRESS_SYMBOL_NUM,
                                                progress_symbols)))
        {
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
        case vi_examine:
            message_symbol(nid, PROGRESS_SYMBOL_NUM, progress_symbols);
            return 1;
        default:
            break;
        }
        break;
        */
    case ni_door:
        switch (vid){
        case vi_look:
        case vi_examine:
            if (Is(achievements_door_opened)){
                message("開いたドアから%nlの触手が見えている。",
                        ni_it);
                return 1;
            }
            break;
        case vi_open:
            if (Is(achievements_door_opened)){
                message("扉はもう開いている。");
            }
            else if (Not(achievements_dog_sitting)){
                message("%nnが邪魔だ。", ni_dog);
            }
            else {
                message("あなたが扉を開けると、\\");
                Set(achievements_door_opened);
                if (Is(achievements_all_completed) &&
                    Not(achievements_magic_circle_erased))
                {
                    msg_tentacle_blocked();
                }
                else {
                    tentacle_attack();
                }
            }
            return 1;
        case vi_close:
            if (Is(achievements_door_opened)){
                message("あなたはなんとか触手を押し出してドアを閉めた。");
                Reset(achievements_door_opened);
            }
            else {
                message("もう閉まっている。");
            }
            return 1;
        default:
            break;
        }
        break;
    case ni_it:
        if (Not(achievements_door_opened)) ReturnNotExist;
        switch (vid){
        case vi_look:
            message("タシヒの触手だ。");
            message("ちなみに「たぶん、しはいしゃの、ひとり」の略である。");
            return 1;
            //        case vi_talk:
        case vi_hit:
        case vi_kick:
        case vi_fight:
        case vi_destroy:
            message("自殺行為はやめておこう。");
            return 1;
        default:
            break;
        }
        break;
    case ni_dog:
        switch (vid){
        case vi_look:
            if (Is(achievements_treasure_chest_opened)){
                message("%nnを前によだれを垂らしている。", ni_chicken);
            }
            else {
                message("扉を守るかのように立ちふさがっている。");
            }
            return 1;
        case vi_talk:
            message("あなたは犬に話しかけた。");
            message("犬はわかったような顔をして聞いている。");
            return 1;
        case vi_touch:
            message("あなたは犬をなでた。");
            message("犬は気持ちよさそうにしている。");
            return 1;
		case vi_push:
		case vi_pull:
        case vi_remove:
        case vi_move:
            message("頑として動かない。");
            return 1;
		case vi_hit:
		case vi_kick:
		case vi_fight:
            message("犬は戦う気がないようだ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_wooden_key:
        if (has_item(nid)){
            switch (vid){
            case vi_use:
				if (Not(achievements_treasure_chest_unlocked)){
					message("宝箱の鍵が開いた。");
					Set(achievements_treasure_chest_unlocked);
					lose_item(nid);
				}
				else {
					message("もう宝箱に鍵は掛かっていない。");
				}
                return 1;
            default:
                break;
            }
        }
        else {
            if (Not(achievements_cabinet_opened) ||
                Is(achievements_treasure_chest_unlocked))
            {
                ReturnNotExist;
            }
            switch (vid){
            case vi_get:
                command_get_item(nid, fi_achievements_wooden_key_taken);
                return 1;
            default:
                break;
            }
        }
		break;
    case ni_sponge:
        if (has_item(nid)){
            switch (vid){
            case vi_use:
				if (Is(achievements_all_completed)){
                    if (Not(achievements_magic_circle_erased)){
        			    message("床をこすると浮かび上がった魔法陣が消えた。");
    					Set(achievements_magic_circle_erased);
					    lose_item(nid);
                        if (Is(achievements_door_opened)){
                            tentacle_attack();
                        }
                    }
                    else {
                        message("魔法陣はもう消えている。");
                    }
                }
                else {
                    message("床をこすると床が綺麗になった気がした。");
                }
                return 1;
            default:
                break;
            }
        }
        else {
            if (Not(achievements_cabinet_opened) ||
                Is(achievements_magic_circle_erased))
            {
                ReturnNotExist;
            }
            switch (vid){
            case vi_get:
                command_get_item(nid, fi_achievements_sponge_taken);
                Set(sponge_wet); // 便宜上、入手時に濡らしておく
                return 1;
            default:
                break;
            }
        }
		break;
    case ni_rubber_duck:
        if (Is(achievements_duck_gone)){
            message("馬鹿め、アヒルは死んだわ！");
            return 1;
        }
        else if (Not(achievements_treasure_chest_opened)){
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
            if (Not(achievements_duck_moved)){
                message("%nnの中に入っている。", ni_treasure_chest);
            }
            else {
                message("%nnの背中に乗っている。", ni_dog);
            }
            break;
        case vi_move:
            if (Not(achievements_duck_moved)){
                message("あなたは%nnを%nnの背中に乗せた",
                        ni_rubber_duck, ni_dog);
                Set(achievements_duck_moved);
            }
            else {
                message("あなたは%nnを%nnの中に戻した",
                        ni_rubber_duck, ni_treasure_chest);
                Reset(achievements_duck_moved);
            }
            return 1;
        default:
            break;
        }
        break;
    case ni_mirror:
        if (Not(achievements_death_completed)) ReturnNotExist;  
        switch (vid){
        case vi_look:
            message("鏡を覗き込むと、忌まわしい死の記憶が見える。");
            return 1;
        case vi_examine:
            message("これは死の支配者の鏡だ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_window:
        if (Not(achievements_alternative_completed)) ReturnNotExist;
        switch (vid){
        case vi_look:
            message("窓に！窓に！");
            message("窓の外には名状し難い異界の様子が見える。");
            return 1;
        case vi_examine:
            message("これは選択肢の支配者の窓だ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_darts_target:
        if (Not(achievements_progress_completed)) ReturnNotExist;
        switch (vid){
        case vi_look:
            message("区切られた領域のあちこちに、"
                    "「クトゥルーの躍り食い」「ティンダロスの猟犬の散歩」"
                    "などと書き込んである。"
                    "罰ゲーム決定用？");
            return 1;
        case vi_examine:
            message("これは進捗の支配者の的だ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_plush_cat:
        if (Not(achievements_complete_completed)) ReturnNotExist;
        switch (vid){
        case vi_look:
            message("ぬいぐるみのふりをしているが、"
                    "じつは旧支配者の一人…%w");
            message("…かもしれない。");
            return 1;
        case vi_examine:
            message("これはやりこみの支配者のぬいぐるみだ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_floor:
        if (Is(achievements_all_completed) &&
            Not(achievements_magic_circle_erased))
        {
            switch (vid){
            case vi_look:
            case vi_examine:
                message("床の上に魔法陣が浮かび上がっている。");
                return 1;
            default:
                break;
            }
            break;
        }
        break;
    case ni_magic_circle:
        if (Not(achievements_all_completed) ||
            Is(achievements_magic_circle_erased))
        {
            ReturnNotExist;
        }
        switch (vid){
        case vi_look:
            message("うさんくさい魔法陣だ。");
            return 1;
        case vi_examine:
            message("これはすべての支配者の魔法陣だ。");
            return 1;
        default:
            break;
        }
        break;
    case ni_chicken:
        if (Not(achievements_treasure_chest_opened)) ReturnNotExist;
        break;
    case ni_ceiling:
    case ni_wall:
        break;
    default:
        *exist = 0;
        // 個別達成度アイテムはまとめて処理
        if (command_symbols(vid, nid, exist)) return 1;
        break;
    }
    return 0;
}

// フラグID配列fidsの(IDがinvalidでない)フラグがすべてONなら非0、
// OFFのフラグが一つでもあれば0を返す
int is_achievements_on(flag_id_t fids[]){
    int i;
    for (i = 0 ; i < SYMBOL_FLAG_NUM ; i++){
        //        printf("is_ach_on %d %d %d\n", i, fids[i], is_flag_off(fids[i]));
        if (fids[i] != fi_invalid && is_flag_off(fids[i])) return 0;
    }
    return 1;
}

int count_achievements(int num, symbol_t symbols[]){
    int i, count = 0;
    for (i = 0 ; i < num ; i++){
        //        printf("%s i=%d\n", symbols[i].message, i);
        // invalidでないフラグでOFFのものが一つでもあれば
        // そのシンボルはカウントしない
        if (is_achievements_on(symbols[i].fids)) count ++;
    }
    return count;
}

void message_active_symbols(int num, symbol_t symbols[]){
    int i, count, outcount = 0;
    count = count_achievements(num, symbols);
    switch (count){
    case 0:
        message("何もない。");
        break;
    default:
        for (i = 0 ; i < num ; i++){
            if (is_achievements_on(symbols[i].fids)){
                outcount ++;
                if (outcount < count-1){
                    message("%nl、\\", symbols[i].nid);
                }
                else if (outcount == count-1){
                    message("%nlと\\", symbols[i].nid);
                }
                else {
                    message("%nlがある。", symbols[i].nid);
                }
            }
        }
        break;
    }
}


void message_achievements_count(char *header, int num, symbol_t symbols[]){
    int count;
    char *hint;

    count = count_achievements(num, symbols);
    hint = achievements_hint(num, symbols);
    if (hint == NULL) hint = "";
    message("なにか書いてある。");
    message("「%s達成率 %d/%d %s」", header, count, num, hint);
}

char *achievements_hint(int num, symbol_t symbols[]){
    int count, ix = 0;

    count = count_achievements(num, symbols);
    if (count == num) return NULL;
    while (ix < num && is_achievements_on(symbols[ix].fids)){
        ix ++;
    }
    assert(ix < num);
    return symbols[ix].hint;
}

flag_id_t *symbol_nid2fids(noun_id_t nid, int num, symbol_t symbols[]){
    int i;
    for (i = 0 ; i < num ; i++){
        if (symbols[i].nid == nid) return symbols[i].fids;
    }
    assert(0);
    return NULL;
}

char *symbol_nid2message(noun_id_t nid, int num, symbol_t symbols[]){
    int i;
    for (i = 0 ; i < num ; i++){
        if (symbols[i].nid == nid) return symbols[i].message;
    }
    assert(0);
    return NULL;
}

void message_symbol(noun_id_t nid, int num, symbol_t symbols[]){
    message("「%s」と刻まれている。",
            symbol_nid2message(nid, num, symbols));
}

void update_achievements_complete_flags(void){
    if (count_achievements(DEATH_SYMBOL_NUM, death_symbols) ==
        DEATH_SYMBOL_NUM)
    {
        Set(achievements_death_completed);
    }
    else {
        Reset(achievements_death_completed);
    }
    if (count_achievements(PROGRESS_SYMBOL_NUM, progress_symbols) ==
        PROGRESS_SYMBOL_NUM)
    {
        Set(achievements_progress_completed);
    }
    else {
        Reset(achievements_progress_completed);
    }
    if (count_achievements(ALTERNATIVE_SYMBOL_NUM, alternative_symbols) ==
        ALTERNATIVE_SYMBOL_NUM)
    {
        Set(achievements_alternative_completed);
    }
    else {
        Reset(achievements_alternative_completed);
    }
    if (count_achievements(COMPLETE_SYMBOL_NUM, complete_symbols) ==
        COMPLETE_SYMBOL_NUM)
    {
        Set(achievements_complete_completed);
    }
    else {
        Reset(achievements_complete_completed);
    }
    if (count_on_flags(4,
                       fi_achievements_death_completed,
                       fi_achievements_progress_completed,
                       fi_achievements_alternative_completed,
                       fi_achievements_complete_completed) == 4)
    {
        Set(achievements_all_completed);
    }
    else {
        Reset(achievements_all_completed);
    }
}


void tentacle_duck(void){
    Reset(achievements_door_opened);
    Set(achievements_duck_gone);
}

void tentacle_attack(void){
    if (Is(achievements_duck_moved) && Not(achievements_duck_gone)){
        msg_tentacle_duck();
        set_wait_return_callback(tentacle_duck);
    }
    else {
        die_tentacle();
    }
}


int command_symbols(verb_id_t vid, noun_id_t nid, int *exist){
    int i, j;
    symbol_t *symbols;
    int num;
    
    for (i = 0 ; i < SYMBOLS_NUM ; i++){
        symbols = symbol_table[i].symbols;
        num = symbol_table[i].num;
        for (j = 0 ; j < num ; j++){
            if (symbols[j].nid == nid){
                if (!is_achievements_on(symbols[j].fids)){
                    ReturnNotExist;
                }
                switch (vid){
                case vi_look:
                    message(symbols[j].description);
                    return 1;
                case vi_examine:
                    message_symbol(nid, num, symbols);
                    return 1;
                default:
                    break;
                }
                *exist = 1;
                return 0;
            }
        }
    }
    return 0;
}
